﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NMeCab.Core
{
    public static class BitUtils
    {
        /// <summary>
        /// 指定範囲のビットフィールド値を取り出す
        /// </summary>
        /// <param name="bits">ビット列を表すUInt32値</param>
        /// <param name="offset">開始ビット位置</param>
        /// <param name="len">ビット長</param>
        /// <returns>ビットフィールド値</returns>
        public static uint GetBitField(uint bits, int offset, int len)
        {
            //BitVector32構造体でもよいが、あえて異なる実装とする。
            uint mask = ~(uint.MaxValue << len);
            return (bits >> offset) & mask;
        }

        /// <summary>
        /// 指定位置のビット値を取り出す
        /// </summary>
        /// <param name="bits">ビット列を表すUInt32値</param>
        /// <param name="offset">ビット位置</param>
        /// <returns>ビット値</returns>
        public static bool GetFlag(uint bits, int offset)
        {
            //BitVector32構造体でもよいが、あえて異なる実装とする。
            uint mask = 0x1u << offset;
            return (bits & mask) != 0;
        }

        /// <summary>
        /// 指定範囲のビット値が一致するか比較する
        /// </summary>
        /// <param name="flags1">ビット列1を表すUInt32値</param>
        /// <param name="flags2">ビット列2を表すUInt32値</param>
        /// <param name="offset">開始ビット位置</param>
        /// <param name="len">ビット長</param>
        /// <returns>比較結果</returns>
        public static bool CompareFlags(uint flags1, uint flags2, int offset, int len)
        {
            uint mask = ~(uint.MaxValue << len) << offset;
            return (flags1 & flags2 & mask) != 0;
        }

        //public static unsafe uint TurnEndianness(uint value) 
        //{
        //    //System.Net.IPAddress.HostToNetworkOrderでもよいが、

        //    byte* pVal = (byte*)&value;

        //    byte* pDist = stackalloc byte[4];
        //    pDist += 3;

        //    *pDist-- = *pVal++;
        //    *pDist-- = *pVal++;
        //    *pDist-- = *pVal++;
        //    *pDist = *pVal;

        //    return *(uint*)pDist;
        //}
    }
}
